# coding=utf-8

from __future__ import unicode_literals
from .. import Provider as AutomotiveProvider


class Provider(AutomotiveProvider):
    # from https://www.revolvy.com/main/index.php?s=Canadian%20licence%20plate%20designs%20and%20serial%20formats
    license_formats = (
        # Alberta
        '???-####',
        # BC
        '??# ##?',
        '?? ####',
        # Manitoba
        '??? ###',
        # New Brunswick
        '??? ###',
        # Newfoundland and Labrador
        '??? ###',
        # NWT
        '######',
        # Nova Scotia
        '??? ###',
        # Nunavut
        '### ###',
        # Ontario
        '### ???',
        '???? ###',
        '??# ###',
        '### #??',
        '?? ####',
        'GV??-###',
        # PEI
        '## ##??',
        # Quebec
        '?## ???',
        # Saskatchewan
        '### ???',
        # Yukon
        '???##'
    )


from __future__ import unicode_literals
import re
from ..en import Provider as AddressProvider


class Provider(AddressProvider):

    postal_code_letters = (
        'A', 'B', 'C', 'E', 'G', 'H', 'J', 'K', 'L', 'M', 'N', 'P', 'R', 'S',
        'T', 'V', 'X', 'Y',
    )

    city_prefixes = ('North', 'East', 'West', 'South', 'New', 'Lake', 'Port')

    city_suffixes = (
        'town', 'ton', 'land', 'ville', 'berg', 'burgh', 'borough', 'bury', 'view', 'port', 'mouth', 'stad', 'furt',
        'chester', 'mouth', 'fort', 'haven', 'side', 'shire')

    building_number_formats = ('#####', '####', '###')

    street_suffixes = (
        'Alley', 'Avenue', 'Branch', 'Bridge', 'Brook', 'Brooks', 'Burg', 'Burgs', 'Bypass', 'Camp', 'Canyon', 'Cape',
        'Causeway', 'Center', 'Centers', 'Circle', 'Circles', 'Cliff', 'Cliffs', 'Club', 'Common', 'Corner', 'Corners',
        'Course', 'Court', 'Courts', 'Cove', 'Coves', 'Creek', 'Crescent', 'Crest', 'Crossing', 'Crossroad', 'Curve',
        'Dale', 'Dam', 'Divide', 'Drive', 'Drive', 'Drives', 'Estate', 'Estates', 'Expressway', 'Extension',
        'Extensions',
        'Fall', 'Falls', 'Ferry', 'Field', 'Fields', 'Flat', 'Flats', 'Ford', 'Fords', 'Forest', 'Forge', 'Forges',
        'Fork',
        'Forks', 'Fort', 'Freeway', 'Garden', 'Gardens', 'Gateway', 'Glen', 'Glens', 'Green', 'Greens', 'Grove',
        'Groves',
        'Harbor', 'Harbors', 'Haven', 'Heights', 'Highway', 'Hill', 'Hills', 'Hollow', 'Inlet', 'Inlet', 'Island',
        'Island',
        'Islands', 'Islands', 'Isle', 'Isle', 'Junction', 'Junctions', 'Key', 'Keys', 'Knoll', 'Knolls', 'Lake',
        'Lakes',
        'Land', 'Landing', 'Lane', 'Light', 'Lights', 'Loaf', 'Lock', 'Locks', 'Locks', 'Lodge', 'Lodge', 'Loop',
        'Mall',
        'Manor', 'Manors', 'Meadow', 'Meadows', 'Mews', 'Mill', 'Mills', 'Mission', 'Mission', 'Motorway', 'Mount',
        'Mountain', 'Mountain', 'Mountains', 'Mountains', 'Neck', 'Orchard', 'Oval', 'Overpass', 'Park', 'Parks',
        'Parkway',
        'Parkways', 'Pass', 'Passage', 'Path', 'Pike', 'Pine', 'Pines', 'Place', 'Plain', 'Plains', 'Plains', 'Plaza',
        'Plaza', 'Point', 'Points', 'Port', 'Port', 'Ports', 'Ports', 'Prairie', 'Prairie', 'Radial', 'Ramp', 'Ranch',
        'Rapid', 'Rapids', 'Rest', 'Ridge', 'Ridges', 'River', 'Road', 'Road', 'Roads', 'Roads', 'Route', 'Row', 'Rue',
        'Run', 'Shoal', 'Shoals', 'Shore', 'Shores', 'Skyway', 'Spring', 'Springs', 'Springs', 'Spur', 'Spurs',
        'Square',
        'Square', 'Squares', 'Squares', 'Station', 'Station', 'Stravenue', 'Stravenue', 'Stream', 'Stream', 'Street',
        'Street', 'Streets', 'Summit', 'Summit', 'Terrace', 'Throughway', 'Trace', 'Track', 'Trafficway', 'Trail',
        'Trail',
        'Tunnel', 'Tunnel', 'Turnpike', 'Turnpike', 'Underpass', 'Union', 'Unions', 'Valley', 'Valleys', 'Via',
        'Viaduct',
        'View', 'Views', 'Village', 'Village', 'Villages', 'Ville', 'Vista', 'Vista', 'Walk', 'Walks', 'Wall', 'Way',
        'Ways', 'Well', 'Wells')

    postal_code_formats = ('?%? %?%', '?%?%?%')

    provinces = (
        'Alberta', 'British Columbia', 'Manitoba', 'New Brunswick',
        'Newfoundland and Labrador', 'Northwest Territories',
        'New Brunswick',  'Nova Scotia', 'Nunavut', 'Ontario',
        'Prince Edward Island', 'Quebec', 'Saskatchewan', 'Yukon Territory')

    provinces_abbr = (
        'AB', 'BC', 'MB', 'NB', 'NL', 'NT', 'NS',
        'NV', 'ON', 'PE', 'QC', 'SK', 'YT')

    city_formats = (
        '{{city_prefix}} {{first_name}}{{city_suffix}}',
        '{{city_prefix}} {{first_name}}',
        '{{first_name}}{{city_suffix}}',
        '{{last_name}}{{city_suffix}}',
    )
    street_name_formats = (
        '{{first_name}} {{street_suffix}}',
        '{{last_name}} {{street_suffix}}'
    )
    street_address_formats = (
        '{{building_number}} {{street_name}}',
        '{{building_number}} {{street_name}} {{secondary_address}}',
    )
    address_formats = (
        "{{street_address}}\n{{city}}, {{province_abbr}} {{postalcode}}",
    )
    secondary_address_formats = ('Apt. ###', 'Suite ###')

    def province(self):
        """
        """
        return self.random_element(self.provinces)

    def province_abbr(self):
        return self.random_element(self.provinces_abbr)

    def city_prefix(self):
        return self.random_element(self.city_prefixes)

    def secondary_address(self):
        return self.numerify(self.random_element(self.secondary_address_formats))

    def postal_code_letter(self):
        """
        Returns a random letter from the list of allowable
        letters in a canadian postal code
        """
        return self.random_element(self.postal_code_letters)

    def postalcode(self):
        """
        Replaces all question mark ('?') occurrences with a random letter
        from postal_code_formats then passes result to
        numerify to insert numbers
        """
        temp = re.sub(r'\?',
            lambda x: self.postal_code_letter(),
            self.random_element(self.postal_code_formats))
        return self.numerify(temp)



# coding=utf-8

from __future__ import unicode_literals
from .. import Provider as AutomotiveProvider


class Provider(AutomotiveProvider):
    # from https://en.wikipedia.org/wiki/Vehicle_registration_plates_of_the_United_Kingdom
    license_formats = (
            '??## ???',
            '??##???'
        )


# coding=utf-8
from __future__ import unicode_literals
from .. import Provider as CompanyProvider


class Provider(CompanyProvider):
    formats = (
        '{{last_name}} {{company_suffix}}',
        '{{last_name}}-{{last_name}} {{company_suffix}}',
        '{{last_name}}, {{last_name}} e {{last_name}} {{company_suffix}}'
    )

    catch_phrase_words = (
        (
            'Abilità', 'Access', 'Adattatore', 'Algoritmo', 'Alleanza', 'Analizzatore', 'Applicazione', 'Approccio',
            'Architettura', 'Archivio', 'Intelligenza artificiale', 'Array', 'Attitudine', 'Benchmark', 'Capacità',
            'Sfida', 'Circuito', 'Collaborazione', 'Complessità', 'Concetto', 'Conglomerato', 'Contingenza', 'Core',
            'Database', 'Data-warehouse', 'Definizione', 'Emulazione', 'Codifica', 'Criptazione', 'Firmware',
            'Flessibilità', 'Previsione', 'Frame', 'framework', 'Funzione', 'Funzionalità', 'Interfaccia grafica',
            'Hardware', 'Help-desk', 'Gerarchia', 'Hub', 'Implementazione', 'Infrastruttura', 'Iniziativa',
            'Installazione', 'Set di istruzioni', 'Interfaccia', 'Soluzione internet', 'Intranet', 'Conoscenza base',
            'Matrici', 'Matrice', 'Metodologia', 'Middleware', 'Migrazione', 'Modello', 'Moderazione', 'Monitoraggio',
            'Moratoria', 'Rete', 'Architettura aperta', 'Sistema aperto', 'Orchestrazione', 'Paradigma', 'Parallelismo',
            'Policy', 'Portale', 'Struttura di prezzo', 'Prodotto', 'Produttività', 'Progetto', 'Proiezione',
            'Protocollo', 'Servizio clienti', 'Software', 'Soluzione', 'Standardizzazione', 'Strategia', 'Struttura',
            'Successo', 'Sovrastruttura', 'Supporto', 'Sinergia', 'Task-force', 'Finestra temporale', 'Strumenti',
            'Utilizzazione', 'Sito web', 'Forza lavoro'
        ),
        (
            'adattiva', 'avanzata', 'migliorata', 'assimilata', 'automatizzata', 'bilanciata', 'centralizzata',
            'compatibile', 'configurabile', 'cross-platform', 'decentralizzata', 'digitalizzata', 'distribuita',
            'piccola', 'ergonomica', 'esclusiva', 'espansa', 'estesa', 'configurabile', 'fondamentale', 'orizzontale',
            'implementata', 'innovativa', 'integrata', 'intuitiva', 'inversa', 'gestita', 'obbligatoria', 'monitorata',
            'multi-canale', 'multi-laterale', 'open-source', 'operativa', 'ottimizzata', 'organica', 'persistente',
            'polarizzata', 'proattiva', 'programmabile', 'progressiva', 'reattiva', 'riallineata', 'ricontestualizzata',
            'ridotta', 'robusta', 'sicura', 'condivisibile', 'stand-alone', 'switchabile', 'sincronizzata', 'sinergica',
            'totale', 'universale', 'user-friendly', 'versatile', 'virtuale', 'visionaria'
        ),
        (
            '24 ore', '24/7', 'terza generazione', 'quarta generazione', 'quinta generazione', 'sesta generazione',
            'asimmetrica', 'asincrona', 'background', 'bi-direzionale', 'biforcata', 'bottom-line', 'coerente',
            'coesiva', 'composita', 'sensibile al contesto', 'basta sul contesto', 'basata sul contenuto', 'dedicata',
            'didattica', 'direzionale', 'discreta', 'dinamica', 'eco-centrica', 'esecutiva', 'esplicita', 'full-range',
            'globale', 'euristica', 'alto livello', 'olistica', 'omogenea', 'ibrida', 'impattante', 'incrementale',
            'intangibile', 'interattiva', 'intermediaria', 'locale', 'logistica', 'massimizzata', 'metodica',
            'mission-critical', 'mobile', 'modulare', 'motivazionale', 'multimedia', 'multi-tasking', 'nazionale',
            'neutrale', 'nextgeneration', 'non-volatile', 'object-oriented', 'ottima', 'ottimizzante', 'radicale',
            'real-time', 'reciproca', 'regionale', 'responsiva', 'scalabile', 'secondaria', 'stabile', 'statica',
            'sistematica', 'sistemica', 'tangibile', 'terziaria', 'uniforme', 'valore aggiunto'
        )
    )

    bsWords = (
        (
            'partnerships', 'comunità', 'ROI', 'soluzioni', 'e-services', 'nicchie', 'tecnologie', 'contenuti',
            'supply-chains', 'convergenze', 'relazioni', 'architetture', 'interfacce', 'mercati', 'e-commerce',
            'sistemi', 'modelli', 'schemi', 'reti', 'applicazioni', 'metriche', 'e-business', 'funzionalità',
            'esperienze', 'webservices', 'metodologie'
        ),
        (
            'implementate', 'utilizzo', 'integrate', 'ottimali', 'evolutive', 'abilitate', 'reinventate', 'aggregate',
            'migliorate', 'incentivate', 'monetizzate', 'sinergizzate', 'strategiche', 'deploy', 'marchi',
            'accrescitive', 'target', 'sintetizzate', 'spedizioni', 'massimizzate', 'innovazione', 'guida',
            'estensioni', 'generate', 'exploit', 'transizionali', 'matrici', 'ricontestualizzate'
        ),
        (
            'valore aggiunto', 'verticalizzate', 'proattive', 'forti', 'rivoluzionari', 'scalabili', 'innovativi',
            'intuitivi', 'strategici', 'e-business', 'mission-critical', '24/7', 'globali', 'B2B', 'B2C', 'granulari',
            'virtuali', 'virali', 'dinamiche', 'magnetiche', 'web', 'interattive', 'sexy', 'back-end', 'real-time',
            'efficienti', 'front-end', 'distributivi', 'estensibili', 'mondiali', 'open-source', 'cross-platform',
            'sinergiche', 'out-of-the-box', 'enterprise', 'integrate', 'di impatto', 'wireless', 'trasparenti',
            'next-generation', 'cutting-edge', 'visionari', 'plug-and-play', 'collaborative', 'olistiche', 'ricche'
        )
    )

    company_suffixes = ('SPA', 'e figli', 'Group', 's.r.l.')

    def catch_phrase(self):
        """
        :example 'Robust full-range hub'
        """
        result = []
        for word_list in self.catch_phrase_words:
            result.append(self.random_element(word_list))

        return " ".join(result)

    def bs(self):
        """
        :example 'integrate extensible convergence'
        """
        result = []
        for word_list in self.bsWords:
            result.append(self.random_element(word_list))

        return " ".join(result)



# coding=utf-8

from __future__ import unicode_literals
from .. import Provider as InternetProvider


class Provider(InternetProvider):

    free_email_domains = (
        'chello.at', 'gmail.com', 'gmx.at', 'kabsi.at',
    )
    tlds = ('at', 'co.at', 'com', 'net', 'org', )

    replacements = (
        ('ä', 'ae'), ('Ä', 'Ae'),
        ('ö', 'oe'), ('Ö', 'Oe'),
        ('ü', 'ue'), ('Ü', 'Ue'),
        ('ß', 'ss'),
    )



# coding=utf-8

import datetime
from decimal import Decimal
import re

import six
import unicodedata


_re_pattern = re.compile('[^\w\s-]', flags=re.U)
_re_pattern_allow_dots = re.compile('[^\.\w\s-]', flags=re.U)
_re_spaces = re.compile('[-\s]+', flags=re.U)


_PROTECTED_TYPES = six.integer_types + (type(None), float, Decimal,
    datetime.datetime, datetime.date, datetime.time)


def is_protected_type(obj):
    """Determine if the object instance is of a protected type.
    Objects of protected types are preserved as-is when passed to
    force_text(strings_only=True).
    """
    return isinstance(obj, _PROTECTED_TYPES)


def force_text(s, encoding='utf-8', strings_only=False, errors='strict'):
    """
    Similar to smart_text, except that lazy instances are resolved to
    strings, rather than kept as lazy objects.
    If strings_only is True, don't convert (some) non-string-like objects.
    """
    # Handle the common case first for performance reasons.
    if issubclass(type(s), six.text_type):
        return s
    if strings_only and is_protected_type(s):
        return s
    try:
        if not issubclass(type(s), six.string_types):
            if six.PY3:
                if isinstance(s, bytes):
                    s = six.text_type(s, encoding, errors)
                else:
                    s = six.text_type(s)
            elif hasattr(s, '__unicode__'):
                s = six.text_type(s)
            else:
                s = six.text_type(bytes(s), encoding, errors)
        else:
            # Note: We use .decode() here, instead of six.text_type(s, encoding,
            # errors), so that if s is a SafeBytes, it ends up being a
            # SafeText at the end.
            s = s.decode(encoding, errors)
    except UnicodeDecodeError as e:
        if not isinstance(s, Exception):
            raise ValueError(s, *e.args)
        else:
            # If we get to here, the caller has passed in an Exception
            # subclass populated with non-ASCII bytestring data without a
            # working unicode method. Try to handle this without raising a
            # further exception by individually forcing the exception args
            # to unicode.
            s = ' '.join(force_text(arg, encoding, strings_only, errors)
                         for arg in s)
    return s


def slugify(value, allow_dots=False, allow_unicode=False):
    """
    Converts to lowercase, removes non-word characters (alphanumerics and
    underscores) and converts spaces to hyphens. Also strips leading and
    trailing whitespace. Modified to optionally allow dots.

    Adapted from Django 1.9
    """
    if allow_dots:
        pattern = _re_pattern_allow_dots
    else:
        pattern = _re_pattern

    value = force_text(value)
    if allow_unicode:
        value = unicodedata.normalize('NFKC', value)
        value = pattern.sub('', value).strip().lower()
        return _re_spaces.sub('-', value)
    value = unicodedata.normalize('NFKD', value).encode('ascii', 'ignore').decode('ascii')
    value = pattern.sub('', value).strip().lower()
    return _re_spaces.sub('-', value)



# coding=utf-8
from __future__ import unicode_literals
from .. import Provider as InternetProvider


class Provider(InternetProvider):

    free_email_domains = (
        'gmail.com', 'googlemail.com', 'hotmail.com', 'suomi24.fi', 
        'kolumbus.fi', 'luukku.com', 'surffi.net'
    )
    
    tlds = ('com', 'com', 'com', 'fi', 'fi', 'net', 'org')



from __future__ import unicode_literals
from collections import OrderedDict

from ..en import Provider as AddressProvider


class Provider(AddressProvider):
    city_prefixes = ('North', 'East', 'West', 'South', 'New', 'Lake', 'Port')

    city_suffixes = (
        'town', 'ton', 'land', 'ville', 'berg', 'burgh', 'borough', 'bury', 'view', 'port', 'mouth', 'stad', 'furt',
        'chester', 'mouth', 'fort', 'haven', 'side', 'shire')

    building_number_formats = ('#####', '####', '###')

    street_suffixes = (
        'Alley', 'Avenue', 'Branch', 'Bridge', 'Brook', 'Brooks', 'Burg', 'Burgs', 'Bypass', 'Camp', 'Canyon', 'Cape',
        'Causeway', 'Center', 'Centers', 'Circle', 'Circles', 'Cliff', 'Cliffs', 'Club', 'Common', 'Corner', 'Corners',
        'Course', 'Court', 'Courts', 'Cove', 'Coves', 'Creek', 'Crescent', 'Crest', 'Crossing', 'Crossroad', 'Curve',
        'Dale', 'Dam', 'Divide', 'Drive', 'Drive', 'Drives', 'Estate', 'Estates', 'Expressway', 'Extension',
        'Extensions',
        'Fall', 'Falls', 'Ferry', 'Field', 'Fields', 'Flat', 'Flats', 'Ford', 'Fords', 'Forest', 'Forge', 'Forges',
        'Fork',
        'Forks', 'Fort', 'Freeway', 'Garden', 'Gardens', 'Gateway', 'Glen', 'Glens', 'Green', 'Greens', 'Grove',
        'Groves',
        'Harbor', 'Harbors', 'Haven', 'Heights', 'Highway', 'Hill', 'Hills', 'Hollow', 'Inlet', 'Inlet', 'Island',
        'Island',
        'Islands', 'Islands', 'Isle', 'Isle', 'Junction', 'Junctions', 'Key', 'Keys', 'Knoll', 'Knolls', 'Lake',
        'Lakes',
        'Land', 'Landing', 'Lane', 'Light', 'Lights', 'Loaf', 'Lock', 'Locks', 'Locks', 'Lodge', 'Lodge', 'Loop',
        'Mall',
        'Manor', 'Manors', 'Meadow', 'Meadows', 'Mews', 'Mill', 'Mills', 'Mission', 'Mission', 'Motorway', 'Mount',
        'Mountain', 'Mountain', 'Mountains', 'Mountains', 'Neck', 'Orchard', 'Oval', 'Overpass', 'Park', 'Parks',
        'Parkway',
        'Parkways', 'Pass', 'Passage', 'Path', 'Pike', 'Pine', 'Pines', 'Place', 'Plain', 'Plains', 'Plains', 'Plaza',
        'Plaza', 'Point', 'Points', 'Port', 'Port', 'Ports', 'Ports', 'Prairie', 'Prairie', 'Radial', 'Ramp', 'Ranch',
        'Rapid', 'Rapids', 'Rest', 'Ridge', 'Ridges', 'River', 'Road', 'Road', 'Roads', 'Roads', 'Route', 'Row', 'Rue',
        'Run', 'Shoal', 'Shoals', 'Shore', 'Shores', 'Skyway', 'Spring', 'Springs', 'Springs', 'Spur', 'Spurs',
        'Square',
        'Square', 'Squares', 'Squares', 'Station', 'Station', 'Stravenue', 'Stravenue', 'Stream', 'Stream', 'Street',
        'Street', 'Streets', 'Summit', 'Summit', 'Terrace', 'Throughway', 'Trace', 'Track', 'Trafficway', 'Trail',
        'Trail',
        'Tunnel', 'Tunnel', 'Turnpike', 'Turnpike', 'Underpass', 'Union', 'Unions', 'Valley', 'Valleys', 'Via',
        'Viaduct',
        'View', 'Views', 'Village', 'Village', 'Villages', 'Ville', 'Vista', 'Vista', 'Walk', 'Walks', 'Wall', 'Way',
        'Ways', 'Well', 'Wells')

    postcode_formats = ('#####', '#####-####')

    states = (
        'Alabama', 'Alaska', 'Arizona', 'Arkansas', 'California', 'Colorado',
        'Connecticut', 'Delaware', 'Florida', 'Georgia', 'Hawaii', 'Idaho',
        'Illinois', 'Indiana', 'Iowa', 'Kansas', 'Kentucky', 'Louisiana',
        'Maine', 'Maryland', 'Massachusetts', 'Michigan', 'Minnesota',
        'Mississippi', 'Missouri', 'Montana', 'Nebraska', 'Nevada',
        'New Hampshire', 'New Jersey', 'New Mexico', 'New York',
        'North Carolina', 'North Dakota', 'Ohio', 'Oklahoma', 'Oregon',
        'Pennsylvania', 'Rhode Island', 'South Carolina', 'South Dakota',
        'Tennessee', 'Texas', 'Utah', 'Vermont', 'Virginia', 'Washington',
        'West Virginia', 'Wisconsin', 'Wyoming',
    )
    states_abbr = (
        'AL', 'AK', 'AS', 'AZ', 'AR', 'CA', 'CO', 'CT', 'DE', 'DC', 'FM', 'FL',
        'GA', 'GU', 'HI', 'ID', 'IL', 'IN', 'IA', 'KS', 'KY', 'LA', 'ME', 'MH',
        'MD', 'MA', 'MI', 'MN', 'MS', 'MO', 'MT', 'NE', 'NV', 'NH', 'NJ', 'NM',
        'NY', 'NC', 'ND', 'MP', 'OH', 'OK', 'OR', 'PW', 'PA', 'PR', 'RI', 'SC',
        'SD', 'TN', 'TX', 'UT', 'VT', 'VI', 'VA', 'WA', 'WV', 'WI', 'WY',
    )

    military_state_abbr = ('AE', 'AA', 'AP')

    military_ship_prefix = ('USS', 'USNS', 'USNV', 'USCGC')

    military_apo_format = ("PSC ####, Box ####")

    military_dpo_format = ("Unit #### Box ####")

    city_formats = (
        '{{city_prefix}} {{first_name}}{{city_suffix}}',
        '{{city_prefix}} {{first_name}}',
        '{{first_name}}{{city_suffix}}',
        '{{last_name}}{{city_suffix}}',
    )

    street_name_formats = (
        '{{first_name}} {{street_suffix}}',
        '{{last_name}} {{street_suffix}}'
    )

    street_address_formats = (
        '{{building_number}} {{street_name}}',
        '{{building_number}} {{street_name}} {{secondary_address}}',
    )

    address_formats = (
        "{{street_address}}\n{{city}}, {{state_abbr}} {{postcode}}",
    )

    address_formats = OrderedDict((
        ("{{street_address}}\n{{city}}, {{state_abbr}} {{postcode}}", 25),
        #  military address formatting.
        ("{{military_apo}}\nAPO {{military_state}} {{postcode}}", 1),
        ("{{military_ship}} {{last_name}}\nFPO {{military_state}} {{postcode}}", 1),
        ("{{military_dpo}}\nDPO {{military_state}} {{postcode}}", 1),
    ))

    secondary_address_formats = ('Apt. ###', 'Suite ###')

    def city_prefix(self):
        return self.random_element(self.city_prefixes)

    def secondary_address(self):
        return self.numerify(self.random_element(self.secondary_address_formats))

    def state(self):
        return self.random_element(self.states)

    def state_abbr(self):
        return self.random_element(self.states_abbr)

    def zipcode(self):
        return "%05d" % self.generator.random.randint(501, 99950)

    def zipcode_plus4(self):
        return "%s-%04d" % (self.zipcode(), self.generator.random.randint(1, 9999))

    def military_ship(self):
        """
        :example 'USS'
        """
        return self.random_element(self.military_ship_prefix)

    def military_state(self):
        """
        :example 'APO'
        """
        return self.random_element(self.military_state_abbr)

    def military_apo(self):
        """
        :example 'PSC 5394 Box 3492
        """
        return self.numerify(self.military_apo_format)

    def military_dpo(self):
        """
        :example 'Unit 3333 Box 9342'
        """
        return self.numerify(self.military_dpo_format)

    # Aliases
    def postalcode(self):
        return self.zipcode()

    def postalcode_plus4(self):
        return self.zipcode_plus4()



# coding=utf-8
from __future__ import unicode_literals
from .. import Provider as AddressProvider


class Provider(AddressProvider):
    """
    Korean Address Provider
    =======================

    Korea has two address and postal code system.

    Address
    -------

    - Address based on land parcel numbers
      (지번 주소, OLD, but someone use consistently)
    - Address based on road names and building numbers (도로명 주소, NEW)

    :meth:`land_address` generate Address based on land parcel numbers and
    :meth:`road_address` generate Address based on road names and building
    numbers.

    Postal code
    -----------

    - Old postal code (6-digit, OLD and dead)
    - New postal code (5-digit, New)

    :meth:`old_postal_code` and :meth:`postcode` generate old 6-digit code
    and :meth:`postal_code` generate newer 5-digit code.

    Reference
    ---------

    - `Official Confirmation Prividing that Old and New Addresses are Identical`__
      (warn: cert error)

    __ https://www.juso.go.kr/addridentity/AddrIdentityHelp.htm

    """

    building_suffixes = (
        '빌라',
        '아파트',
        '연립',
        '마을',
        '타운',
        '타워',
    )
    road_suffixes = ('로', '길', '거리', '가')
    town_suffixes = ('동', '리', '마을')
    postcode_formats = ('###-###',)
    new_postal_code_formats = ('#####',)
    metropolitan_cities = (
        '서울특별시',
        '부산광역시',
        '대구광역시',
        '인천광역시',
        '광주광역시',
        '대전광역시',
        '울산광역시',
        '세종특별자치시',
    )
    provinces = (
        '경기도',
        '강원도',
        '충청북도',
        '충청남도',
        '전라북도',
        '전라남도',
        '경상북도',
        '경상남도',
        '제주특별자치도',
    )
    cities = (
        '파주시',
        '수원시',
        '수원시 권선구',
        '수원시 팔달구',
        '수원시 영통구',
        '성남시',
        '성남시 수정구',
        '성남시 중원구',
        '화성시',
        '성남시 분당구',
        '안양시',
        '안양시 만안구',
        '안양시 동안구',
        '부천시',
        '부천시 원미구',
        '부천시 소사구',
        '부천시 오정구',
        '광명시',
        '평택시',
        '이천시',
        '동두천시',
        '안산시',
        '안산시 상록구',
        '안산시 단원구',
        '안성시',
        '고양시',
        '고양시 덕양구',
        '고양시 일산동구',
        '고양시 일산서구',
        '과천시',
        '구리시',
        '남양주시',
        '오산시',
        '시흥시',
        '군포시',
        '의왕시',
        '하남시',
        '김포시',
        '용인시',
        '용인시 처인구',
        '용인시 기흥구',
        '용인시 수지구',
        '연천군',
        '가평군',
        '양평군',
        '광주시',
        '포천시',
        '양주시',
        '수원시 장안구',
        '의정부시',
        '여주시',
    )
    road_names = (
        '압구정',
        '도산대',
        '학동',
        '봉은사',
        '테헤란',
        '역삼',
        '논현',
        '언주',
        '강남대',
        '양재천',
        '삼성',
        '영동대',
        '개포',
        '선릉',
        '반포대',
        '서초중앙',
        '서초대',
        '잠실',
        '석촌호수',
        '백제고분',
        '가락',
        '오금',
    )
    boroughs = (
        '종로구',
        '중구',
        '용산구',
        '성동구',
        '광진구',
        '동대문구',
        '중랑구',
        '성북구',
        '강북구',
        '도봉구',
        '노원구',
        '은평구',
        '서대문구',
        '마포구',
        '양천구',
        '강서구',
        '구로구',
        '금천구',
        '영등포구',
        '동작구',
        '관악구',
        '서초구',
        '강남구',
        '송파구',
        '강동구',
        '동구',
        '서구',
        '남구',
        '북구',
    )
    countries = ('가나', '가봉', '가이아나', '감비아', '과테말라', '그레나다', '그리스', '기니', '기니비사우',
                 '나미비아', '나우루', '나이지리아', '남수단', '남아프리카 공화국', '네덜란드 왕국', '네팔',
                 '노르웨이', '뉴질랜드', '니제르', '니카라과', '대한민국', '덴마크', '도미니카 공화국',
                 '도미니카 연방', '독일', '동티모르', '라오스', '라이베리아', '라트비아', '러시아', '레바논',
                 '레소토', '루마니아', '룩셈부르크', '르완다', '리비아', '리투아니아', '리히텐슈타인',
                 '마다가스카르', '마셜 제도', '마케도니아 공화국', '말라위', '말레이시아', '말리', '멕시코',
                 '모나코', '모로코', '모리셔스', '모리타니', '모잠비크', '몬테네그로', '몰도바', '몰디브',
                 '몰타', '몽골', '미국', '미얀마', '미크로네시아 연방', '바누아투', '바레인', '바베이도스',
                 '바하마', '방글라데시', '베냉', '베네수엘라', '베트남', '벨기에', '벨라루스', '벨리즈',
                 '보스니아 헤르체고비나', '보츠와나', '볼리비아', '부룬디', '부르키나파소', '부탄', '불가리아',
                 '브라질', '브루나이', '사모아', '사우디아라비아', '산마리노', '상투메 프린시페', '세네갈',
                 '세르비아', '세이셸', '세인트루시아', '세인트빈센트 그레나딘', '세인트키츠 네비스',
                 '소말리아', '솔로몬 제도', '수단', '수리남', '스리랑카', '스와질란드', '스웨덴', '스위스',
                 '스페인', '슬로바키아', '슬로베니아', '시리아', '시에라리온 공화국', '싱가포르',
                 '아랍에미리트', '아르메니아', '아르헨티나', '아이슬란드', '아이티', '아일랜드',
                 '아제르바이잔', '아프가니스탄', '안도라', '알바니아', '알제리', '앙골라', '앤티가 바부다',
                 '에리트레아', '에스토니아', '에콰도르', '에티오피아', '엘살바도르', '영국', '예멘', '오만',
                 '오스트레일리아', '오스트리아', '온두라스', '요르단', '우간다', '우루과이', '우즈베키스탄',
                 '우크라이나', '이라크', '이란', '이스라엘', '이집트', '이탈리아', '인도네시아', '일본',
                 '자메이카', '잠비아', '적도 기니', '조선민주주의인민공화국', '조지아', '중앙아프리카 공화국',
                 '중화인민공화국', '지부티', '짐바브웨', '차드', '체코', '칠레', '카메룬', '카보베르데',
                 '카자흐스탄', '카타르', '캄보디아', '캐나다', '케냐', '코모로', '코스타리카', '코트디부아르',
                 '콜롬비아', '콩고 공화국', '콩고 민주 공화국', '쿠바', '쿠웨이트', '크로아티아',
                 '키르기스스탄', '키리바시', '키프로스', '타이', '타지키스탄', '탄자니아', '터키',
                 '토고', '통가', '투르크메니스탄', '투발루', '튀니지', '트리니다드 토바고', '파나마',
                 '파라과이', '파키스탄', '파푸아 뉴기니', '팔라우', '페루', '포르투갈', '폴란드', '프랑스',
                 '피지', '핀란드', '필리핀', '헝가리'
    )
    building_dongs = (
        '가',
        '나',
        '다',
        '라',
        '마',
        '바',
        '##',
        '###',
    )
    land_numbers = (
        '###',
        '###-#',
        '###-##',
    )
    road_numbers = (
        '#',
        '##',
        '###',
    )

    town_formats = (
        '{{first_name}}{{last_name}}{{town_suffix}}',
        '{{first_name}}{{last_name}}{{last_name}}{{town_suffix}}',
    )
    building_name_formats = (
        '{{first_name}}{{last_name}}{{building_suffix}}',
        '{{first_name}}{{last_name}}{{last_name}}{{building_suffix}}',
    )
    address_detail_formats = (
        '{{building_name}}',
        '{{building_name}} ###호',
        '{{building_name}} {{building_dong}}동 ###호',
    )
    road_formats = (
        '{{road_name}}{{road_suffix}}',
        '{{road_name}}{{road_number}}{{road_suffix}}',
    )
    road_address_formats = (
        '{{metropolitan_city}} {{borough}} {{road}}',
        '{{province}} {{city}} {{road}}',
        '{{metropolitan_city}} {{borough}} {{road}} ({{town}})',
        '{{province}} {{city}} {{road}} ({{town}})',
    )
    land_address_formats = (
        '{{metropolitan_city}} {{borough}} {{town}} {{land_number}}',
        '{{province}} {{city}} {{town}} {{land_number}}',
    )

    # Keep backward compatibility
    city_suffixes = ('시',)
    street_suffixes = road_suffixes
    street_name_formats = ('{{road_name}}',)
    street_address_formats = road_address_formats
    address_formats = road_address_formats

    def land_number(self):
        """
        :example 507
        """
        return self.bothify(self.random_element(self.land_numbers))

    def land_address(self):
        """
        :example 세종특별자치시 어진동 507
        """
        pattern = self.random_element(self.land_address_formats)
        return self.generator.parse(pattern)

    def road_number(self):
        """
        :example 24
        """
        return self.bothify(self.random_element(self.road_numbers))

    def road_address(self):
        """
        :example 세종특별자치시 도움5로 19 (어진동)
        """
        pattern = self.random_element(self.road_address_formats)
        return self.generator.parse(pattern)

    def address_detail(self):
        """
        :example 가나아파트 가동 102호
        """
        pattern = self.bothify(self.random_element(
            self.address_detail_formats))
        return self.generator.parse(pattern)

    def road(self):
        """
        :example 도움5로
        """
        pattern = self.random_element(self.road_formats)
        return self.generator.parse(pattern)

    def road_name(self):
        """
        :example 압구정
        """
        return self.random_element(self.road_names)

    def road_suffix(self):
        """
        :example 길
        """
        return self.random_element(self.road_suffixes)

    def metropolitan_city(self):
        """
        :example 서울특별시
        """
        return self.random_element(self.metropolitan_cities)

    def province(self):
        """
        :example 경기도
        """
        return self.random_element(self.provinces)

    def city(self):
        """
        :example 고양시
        """
        pattern = self.random_element(self.cities)
        return self.generator.parse(pattern)

    def borough(self):
        """
        :example 중구
        """
        return self.random_element(self.boroughs)

    def town(self):
        """
        :example 가나동
        """
        pattern = self.random_element(self.town_formats)
        return self.generator.parse(pattern)

    def town_suffix(self):
        """
        :example 동
        """
        return self.random_element(self.town_suffixes)

    def building_name(self):
        """
        :example 김구아파트
        """
        pattern = self.random_element(self.building_name_formats)
        return self.generator.parse(pattern)

    def building_suffix(self):
        """
        :example 아파트
        """
        return self.random_element(self.building_suffixes)

    def building_dong(self):
        """
        :example 가
        """
        return self.bothify(self.random_element(self.building_dongs))

    def old_postal_code(self):
        """
        :example 123-456
        """
        return self.bothify(self.random_element(self.postcode_formats))

    def postal_code(self):
        """
        :example 12345
        """
        return self.bothify(self.random_element(self.new_postal_code_formats))



# coding=utf-8
from __future__ import unicode_literals
from .. import Provider as PersonProvider

class Provider(PersonProvider):
    formats = (
        '{{first_name}} {{last_name}}',
        '{{first_name}} {{last_name}}',
        '{{first_name}} {{last_name}}',
        '{{first_name}} {{last_name}}',
        '{{first_name}} {{last_name}}',
        '{{first_name}} {{last_name}}',
        '{{first_name}} {{last_name}}',
        '{{first_name}} {{last_name}}',
        '{{first_name}} {{last_name}}',
        '{{first_name}} {{last_name}}',
        '{{first_name}} {{last_name}}',
        '{{first_name}} {{last_name}}',
        '{{first_name}} {{last_name}}',
        '{{first_name}} {{last_name}}',
        '{{first_name}} {{last_name}}',
        '{{first_name}} {{last_name}}',
        '{{first_name_male}}-{{first_name_male}} {{last_name}}',
        '{{first_name_male}}-{{first_name_male}} {{last_name}}',
        '{{first_name_female}}-{{first_name_female}} {{last_name}}',
        '{{first_name_female}}-{{first_name_female}} {{last_name}}',
        '{{first_name}} {{last_name}}-{{last_name}}',
        '{{first_name}} {{last_name}}-{{last_name}}',
        '{{prefix}} {{first_name_male}} {{last_name}}',
    )

    # 100 most common male first names, alphabetically.
    # Source: http://www.ssb.no/a/navn/fornavn-menn-100.html
    first_names_male = (
        'Adrian',
        'Alexander',
        'Alf',
        'Anders',
        'Andreas',
        'Arild',
        'Arne',
        'Asbjørn',
        'Bjørn',
        'Christian',
        'Dag',
        'Daniel',
        'Egil',
        'Einar',
        'Eirik',
        'Eivind',
        'Emil',
        'Erik',
        'Erling',
        'Espen',
        'Finn',
        'Frank',
        'Fredrik',
        'Frode',
        'Geir',
        'Gunnar',
        'Hans',
        'Harald',
        'Helge',
        'Henrik',
        'Håkon',
        'Håvard',
        'Ivar',
        'Jan',
        'Jens',
        'Joakim',
        'Johannes',
        'Johan',
        'John',
        'Jonas',
        'Jon',
        'Jørgen',
        'Karl',
        'Kenneth',
        'Kim',
        'Kjell',
        'Kjetil',
        'Knut',
        'Kåre',
        'Kristian',
        'Kristoffer',
        'Lars',
        'Leif',
        'Magne',
        'Magnus',
        'Marius',
        'Markus',
        'Martin',
        'Mathias',
        'Morten',
        'Nils',
        'Odd',
        'Ola',
        'Olav',
        'Ole',
        'Per',
        'Petter',
        'Pål',
        'Roar',
        'Robert',
        'Roger',
        'Rolf',
        'Roy',
        'Rune',
        'Sander',
        'Sebastian',
        'Sigurd',
        'Simen',
        'Sindre',
        'Sondre',
        'Steinar',
        'Stein',
        'Stian',
        'Stig',
        'Svein',
        'Sverre',
        'Terje',
        'Thomas',
        'Thor',
        'Tobias',
        'Tommy',
        'Tom',
        'Torbjørn',
        'Tore',
        'Tor',
        'Trond',
        'Vegard',
        'Vidar',
        'Øystein',
        'Øyvind',
    )

    # 100 most common female first names, alphabetically.
    # Source: http://www.ssb.no/a/navn/fornavn-kvinner-100.html
    first_names_female = (
        'Andrea',
        'Anette',
        'Anita',
        'Anna',
        'Anne',
        'Ann',
        'Astrid',
        'Aud',
        'Bente',
        'Berit',
        'Bjørg',
        'Britt',
        'Camilla',
        'Cathrine',
        'Cecilie',
        'Elin',
        'Elisabeth',
        'Elise',
        'Eli',
        'Ellen',
        'Else',
        'Emilie',
        'Emma',
        'Eva',
        'Gerd',
        'Grete',
        'Grethe',
        'Gro',
        'Gunn',
        'Hanna',
        'Hanne',
        'Hege',
        'Heidi',
        'Helene',
        'Hilde',
        'Ida',
        'Ingeborg',
        'Inger',
        'Ingrid',
        'Irene',
        'Janne',
        'Jenny',
        'Jorunn',
        'Julie',
        'Karen',
        'Karin',
        'Kari',
        'Karoline',
        'Kirsten',
        'Kjersti',
        'Kristine',
        'Kristin',
        'Laila',
        'Lene',
        'Linda',
        'Line',
        'Linn',
        'Lise',
        'Liv',
        'Malin',
        'Maren',
        'Marianne',
        'Maria',
        'Marie',
        'Mari',
        'Marit',
        'Marte',
        'Martine',
        'May',
        'Mette',
        'Mona',
        'Monica',
        'Nina',
        'Nora',
        'Ragnhild',
        'Randi',
        'Reidun',
        'Rita',
        'Ruth',
        'Sara',
        'Sigrid',
        'Silje',
        'Siri',
        'Sissel',
        'Siv',
        'Sofie',
        'Solveig',
        'Stine',
        'Synnøve',
        'Thea',
        'Tone',
        'Tonje',
        'Torill',
        'Tove',
        'Trine',
        'Turid',
        'Unni',
        'Vilde',
        'Wenche',
        'Åse',
    )

    first_names = first_names_male + first_names_female

    # 100 most common last names, alphabetically.
    # Source: http://www.ssb.no/a/navn/alf/etter100.html
    last_names = (
        'Aasen',
        'Aas',
        'Abrahamsen',
        'Ahmed',
        'Ali',
        'Amundsen',
        'Andersen',
        'Andreassen',
        'Andresen',
        'Antonsen',
        'Arnesen',
        'Aune',
        'Bakken',
        'Bakke',
        'Berge',
        'Berg',
        'Berntsen',
        'Bøe',
        'Birkeland',
        'Brekke',
        'Christensen',
        'Dahl',
        'Danielsen',
        'ødegård',
        'Edvardsen',
        'Eide',
        'Eliassen',
        'Ellingsen',
        'Engen',
        'Eriksen',
        'Evensen',
        'Fredriksen',
        'Gulbrandsen',
        'Gundersen',
        'Hagen',
        'Halvorsen',
        'Hansen',
        'Hanssen',
        'Haugen',
        'Hauge',
        'Haugland',
        'Haug',
        'Helland',
        'Henriksen',
        'Holm',
        'Isaksen',
        'Iversen',
        'Jacobsen',
        'Jakobsen',
        'Jensen',
        'Jenssen',
        'Johannessen',
        'Johansen',
        'Johnsen',
        'Jørgensen',
        'Karlsen',
        'Knudsen',
        'Knutsen',
        'Kristensen',
        'Kristiansen',
        'Kristoffersen',
        'Larsen',
        'Lien',
        'Lie',
        'Lunde',
        'Lund',
        'Madsen',
        'Martinsen',
        'Mathisen',
        'Mikkelsen',
        'Moen',
        'Moe',
        'Myhre',
        'Myklebust',
        'Nguyen',
        'Nielsen',
        'Nilsen',
        'Næss',
        'Nygård',
        'Olsen',
        'Paulsen',
        'Pedersen',
        'Pettersen',
        'Rasmussen',
        'Rønning',
        'Ruud',
        'Sandvik',
        'Simonsen',
        'Sivertsen',
        'Solberg',
        'Solheim',
        'Sørensen',
        'Sæther',
        'Strand',
        'Strøm',
        'Svendsen',
        'Tangen',
        'Thomassen',
        'Thorsen',
        'Tveit',
        'Vik',
    )

    prefixes = (
        'Dr.', 'Prof.',
    )





# coding: utf-8

from __future__ import unicode_literals
from .. import Provider as DateTimeProvider

class Provider(DateTimeProvider):

    def day_of_week(self):
        day = self.date('%w')
        DAY_NAMES = {
            "0": "hétfő",
            "1": "kedd",
            "2": "szerda",
            "3": "csütörtök",
            "4": "péntek",
            "5": "szombat",
            "6": "vasárnap",
        }

        return DAY_NAMES[day]

    def month_name(self):
        month = self.month()
        MONTH_NAMES = {
            "01": "január",
            "02": "február",
            "03": "március",
            "04": "április",
            "05": "május",
            "06": "junius",
            "07": "julius",
            "08": "augusztus",
            "09": "szeptember",
            "10": "október",
            "11": "november",
            "12": "december",
        }

        return MONTH_NAMES[month]



localized = True

from .. import BaseProvider


class Provider(BaseProvider):
    formats = ('###-###-###',)

    msisdn_formats = (
        '#############',
    )

    def phone_number(self):
        return self.numerify(self.random_element(self.formats))

    def msisdn(self):
        """ https://en.wikipedia.org/wiki/MSISDN """
        return self.numerify(self.random_element(self.msisdn_formats))



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from .. import Provider as LoremProvider


class Provider(LoremProvider):
    word_connector = ''
    word_list = (
        u'一個',
        u'我們',
        u'時間',
        u'台灣',
        u'可以',
        u'公司',
        u'沒有',
        u'信息',
        u'下載',
        u'軟體',
        u'注冊',
        u'自己',
        u'產品',
        u'工作',
        u'論壇',
        u'企業',
        u'這個',
        u'他們',
        u'管理',
        u'已經',
        u'問題',
        u'內容',
        u'使用',
        u'進行',
        u'市場',
        u'服務',
        u'如果',
        u'系統',
        u'技術',
        u'發展',
        u'現在',
        u'作者',
        u'就是',
        u'網絡',
        u'提供',
        u'相關',
        u'我的',
        u'文章',
        u'方式',
        u'電話',
        u'發表',
        u'所有',
        u'時候',
        u'因為',
        u'台北',
        u'有限',
        u'公司',
        u'什麼',
        u'還是',
        u'開始',
        u'本站',
        u'發布',
        u'自己',
        u'支持',
        u'在線',
        u'國家',
        u'生活',
        u'聯系',
        u'積分',
        u'主題',
        u'所以',
        u'不能',
        u'的人',
        u'上海',
        u'中心',
        u'世界',
        u'游戲',
        u'需要',
        u'價格',
        u'用戶',
        u'通過',
        u'要求',
        u'不是',
        u'免費',
        u'個人',
        u'但是',
        u'地址',
        u'網站',
        u'情況',
        u'最后',
        u'設計',
        u'同時',
        u'這些',
        u'活動',
        u'手機',
        u'推薦',
        u'一些',
        u'主要',
        u'大家',
        u'發現',
        u'目前',
        u'文件',
        u'你的',
        u'不過',
        u'評論',
        u'生產',
        u'美國',
        u'圖片',
        u'經濟',
        u'功能',
        u'國際',
        u'的是',
        u'選擇',
        u'其他',
        u'這樣',
        u'會員',
        u'環境',
        u'來自',
        u'日期',
        u'成為',
        u'他的',
        u'最新',
        u'專業',
        u'一下',
        u'人員',
        u'任何',
        u'教育',
        u'資料',
        u'狀態',
        u'都是',
        u'點擊',
        u'為了',
        u'不會',
        u'出現',
        u'知道',
        u'社會',
        u'名稱',
        u'而且',
        u'介紹',
        u'音樂',
        u'等級',
        u'可能',
        u'這種',
        u'建設',
        u'朋友',
        u'雖然',
        u'電子',
        u'資源',
        u'看到',
        u'精華',
        u'電影',
        u'如何',
        u'新聞',
        u'閱讀',
        u'安全',
        u'全國',
        u'隻有',
        u'回復',
        u'大學',
        u'學生',
        u'學習',
        u'關於',
        u'項目',
        u'不同',
        u'以及',
        u'有關',
        u'那麼',
        u'開發',
        u'還有',
        u'隻是',
        u'非常',
        u'研究',
        u'廣告',
        u'首頁',
        u'方法',
        u'希望',
        u'地方',
        u'也是',
        u'單位',
        u'怎麼',
        u'應該',
        u'今天',
        u'以上',
        u'更新',
        u'帖子',
        u'顯示',
        u'能力',
        u'電腦',
        u'記者',
        u'查看',
        u'位置',
        u'不要',
        u'由於',
        u'無法',
        u'詳細',
        u'投資',
        u'是一',
        u'一般',
        u'進入',
        u'發生',
        u'這裡',
        u'感覺',
        u'更多',
        u'你們',
        u'的話',
        u'起來',
        u'標准',
        u'一樣',
        u'認為',
        u'女人',
        u'那個',
        u'設備',
        u'搜索',
        u'之后',
        u'然后',
        u'學校',
        u'銷售',
        u'組織',
        u'說明',
        u'提高',
        u'為什',
        u'作品',
        u'或者',
        u'喜歡',
        u'東西',
        u'方面',
        u'簡介',
        u'必須',
        u'經營',
        u'科技',
        u'作為',
        u'其中',
        u'運行',
        u'工程',
        u'解決',
        u'操作',
        u'經驗',
        u'地區',
        u'重要',
        u'直接',
        u'登錄',
        u'合作',
        u'結果',
        u'影響',
        u'這是',
        u'行業',
        u'對於',
        u'表示',
        u'程序',
        u'包括',
        u'留言',
        u'規定',
        u'處理',
        u'男人',
        u'各種',
        u'部門',
        u'數據',
        u'具有',
        u'商品',
        u'系列',
        u'大小',
        u'因此',
        u'關系',
        u'可是',
        u'比較',
        u'文化',
        u'一直',
        u'法律',
        u'這麼',
        u'您的',
        u'城市',
        u'分析',
        u'基本',
        u'最大',
        u'類別',
        u'兩個',
        u'日本',
        u'得到',
        u'一次',
        u'繼續',
        u'成功',
        u'她的',
        u'責任',
        u'深圳',
        u'業務',
        u'歡迎',
        u'加入',
        u'能夠',
        u'覺得',
        u'部分',
        u'中文',
        u'根據',
        u'人民',
        u'政府',
        u'控制',
        u'其實',
        u'之間',
        u'一種',
        u'威望',
        u'實現',
        u'語言',
        u'出來',
        u'謝謝',
        u'社區',
        u'品牌',
        u'是否',
        u'工具',
        u'完全',
        u'決定',
        u'很多',
        u'網上',
        u'事情',
        u'今年',
        u'國內',
        u'以后',
        u'制作',
        u'瀏覽',
        u'過程',
        u'完成',
        u'類型',
        u'來源',
        u'質量',
        u'有些',
        u'一起',
        u'當然',
        u'汽車',
        u'一點',
        u'幫助',
        u'增加',
        u'歷史',
        u'以下',
        u'不斷',
        u'應用',
        u'那些',
        u'密碼',
        u'計劃',
        u'如此',
        u'次數',
        u'到了',
        u'擁有',
        u'孩子',
        u'原因',
        u'參加',
        u'隻要',
        u'報告',
        u'當前',
        u'客戶',
        u'正在',
        u'注意',
        u'標題',
        u'空間',
        u'一定',
        u'一切',
        u'特別',
        u'全部',
        u'准備',
    )


from __future__ import unicode_literals
from .. import Provider as PhoneNumberProvider


class Provider(PhoneNumberProvider):
    formats = (
        '+977 ##########',
        '+977 ### #######',
        '984#######',
        '985#######',
        '980#######',
    )



# coding=utf-8
from __future__ import unicode_literals

from .. import Provider as PersonProvider


class Provider(PersonProvider):
    formats = (
        '{{first_name_male}} {{last_name_male}}',
        '{{first_name_male}} {{last_name_male}}',
        '{{first_name_male}} {{last_name_male}}',
        '{{first_name_male}} {{last_name_male}}',
        '{{first_name_male}} {{last_name_male}}-{{last_name_male}}',
        '{{first_name_female}} {{last_name_female}}',
        '{{first_name_female}} {{last_name_female}}',
        '{{first_name_female}} {{last_name_female}}',
        '{{first_name_female}} {{last_name_female}}',
        '{{first_name_female}} {{last_name_female}}-{{last_name_female}}',
        '{{prefix_male}} {{first_name_male}} {{last_name_male}}',
        '{{prefix_female}} {{first_name_female}} {{last_name_female}}',
        '{{prefix_male}} {{first_name_male}} {{last_name_male}}',
        '{{prefix_female}} {{first_name_female}} {{last_name_female}}'
    )

    # names from https://en.wikipedia.org/wiki/Ghanaian_name, https://buzzghana.com/ghanaian-names/, https://en.wikipedia.org/wiki/Akan_names, 
    
    first_names_male = (
        'Aaron',
        'Abeiku',
        'Adam',
        'Adrian',
        'Akwesi',
        'Albert',
        'Alex',
        'Alexander',
        'Andrew',
        'Anthony',
        'Antony',
        'Arthur',
        'Ben',
        'Benjamin',
        'Bernard',
        'Bruce',
        'Carl',
        'Charles',
        'Christian',
        'Clifford',
        'Colins',
        'Daniel',
        'Danny',
        'David',
        'Denis',
        'Dennis',
        'Derrick',
        'Dominic',
        'Donald',
        'Douglas',
        'Duncan',
        'Edward',
        'Ekow',
        'Elliot',
        'Elliott',
        'Eric',
        'Fiifi',
        'Francis',
        'Frank',
        'Frederick',
        'George',
        'Gerald',
        'Gordon',
        'Graham',
        'Gregory',
        'Harry',
        'Henry',
        'Howard',
        'Isaac',
        'Akwasi',
        'Jack',
        'Jacob',
        'Jake',
        'James',
        'Jason',
        'Jeffrey',
        'Jeremy',
        'Joe',
        'Joel',
        'John',
        'Jonathan',
        'Joojo',
        'Joseph',
        'Josh',
        'Joshua',
        'Josiah',
        'Julian',
        'Justin',
        'Karl',
        'Kenneth',
        'Kevin',
        'Kofi',
        'Kojo',
        'Kujoe',
        'Kwabena',
        'Kwadwo',
        'Kwaku',
        'Kwame',
        'Kwamena',
        'Kwasi',
        'Kweku',
        'Kwesi',
        'Kyle',
        'Lawrence',
        'Leslie',
        'Louis',
        'Luke',
        'Malcolm',
        'Marcus',
        'Mark',
        'Martin',
        'Mathew',
        'Matthew',
        'Max',
        'Michael',
        'Nathan',
        'Nicholas',
        'Nigel',
        'Oliver',
        'Patrick',
        'Paul',
        'Peter',
        'Philip',
        'Phillip',
        'Raymond',
        'Richard',
        'Robert',
        'Roger',
        'Ronald',
        'Russell',
        'Sam',
        'Samuel',
        'Shaun',
        'Simon',
        'Stanley',
        'Stephen',
        'Steven',
        'Terence',
        'Thomas',
        'Timothy',
        'Tom',
        'Tony',
        'Victor',
        'Vincent',
        'William',
        'Yaw'
    )

    first_names_female = (
        'Aba',
        'Abena',
        'Abigail',
        'Adwoa',
        'Afia',
        'Afua',
        'Akos',
        'Akosua',
        'Akua',
        'Akumaa',
        'Alice',
        'Ama',
        'Amanda',
        'Amber',
        'Amelia',
        'Angela',
        'Ann',
        'Annette',
        'Awesi',
        'Baaba',
        'Barbara',
        'Beatrice',
        'COmfort',
        'Caroline',
        'Catherine',
        'Charlotte',
        'Christina',
        'Comfort',
        'Constance',
        'Danielle',
        'Deborah',
        'Debra',
        'Denise',
        'Dora',
        'Dorcas',
        'Dorothy',
        'Eliabeth',
        'Elizabeth',
        'Emily',
        'Emma',
        'Ernestina',
        'Esi',
        'Eunice',
        'Felicia',
        'Francesca',
        'Gemma',
        'Georgia',
        'Georgina',
        'Gifty',
        'Grace',
        'Grace',
        'Hannabel',
        'Hannah',
        'Harriet',
        'Helen',
        'Irene',
        'Janet',
        'Janet',
        'Janice',
        'Jasmine',
        'Jennifer',
        'Jessica',
        'Jill',
        'Joanna',
        'Josephine',
        'Joyce',
        'Joyce',
        'Judith',
        'Julia',
        'Juliana',
        'Julie',
        'Karen',
        'Kate',
        'Katherine',
        'Katy',
        'Lawrencia',
        'Linda',
        'Lisa',
        'Lorraine',
        'Lucy',
        'Lucy',
        'Lydia',
        'Lydia',
        'Mandy',
        'Margaret',
        'Margaret',
        'Maria',
        'Marian',
        'Marilyn',
        'Mary',
        'Mary',
        'Maureen',
        'Michelle',
        'Millicent',
        'Nana Ama',
        'Naomi',
        'Natalie',
        'Natasha',
        'Nicola',
        'Nimakoah',
        'Olivia',
        'Pamela',
        'Patricia',
        'Paula',
        'Priscilla',
        'Rachael',
        'Rachel',
        'Rebecca',
        'Rebecca',
        'Regina',
        'Rita',
        'Roselyn',
        'Rosemary',
        'Rosemary',
        'Ruth',
        'Salomey',
        'Samantha',
        'Sandra',
        'Sarah',
        'Sarah',
        'Sarah',
        'Sharon',
        'Sheila',
        'Shirley',
        'Stephanie',
        'Susan',
        'Susan',
        'Sylvia',
        'Teresa',
        'Tina',
        'Tracy',
        'Vanessa',
        'Veronica',
        'Victoria',
        'Vida',
        'Wendy',
        'Yaa',
        'Yvonne'
    )

    first_names = first_names_male + first_names_female

    last_names_male = (
        'Acheampong',
        'Adomah',
        'Adomako',
        'Adu',
        'Adusei',
        'Adutwum',
        'Afirifa',
        'Afoakwa',
        'Agyapong',
        'Agyapong',
        'Agyare',
        'Agyei',
        'Agyemang',
        'Ahortor',
        'Akoto',
        'Akowua',
        'Akyeamfuɔ',
        'Akyeampong',
        'Akyena',
        'Akyerεko',
        'Amo',
        'Amoa',
        'Amoako',
        'Amoasi',
        'Ampadu',
        'Ampofo',
        'Amponsah',
        'Andorful',
        'Ankra',
        'Anokye',
        'Ansa',
        'Antwi',
        'Antwi',
        'Appia',
        'Appiah',
        'Asamoa',
        'Asamoah',
        'Asante',
        'Asare',
        'Asenso',
        'Asiama',
        'Asiedu',
        'Ata',
        'Awuah',
        'Baa',
        'Baafi',
        'Baah',
        'Baawia',
        'Badu',
        'Boadi',
        'Boadu',
        'Boahen',
        'Boakye',
        'Boaten',
        'Boateng',
        'Bona',
        'Bonsra',
        'Bonsu',
        'Daako',
        'Danso',
        'Darko',
        'Donkor',
        'Duah',
        'Dwamena',
        'Fofie',
        'Fosu',
        'Gyamfi',
        'Gyasi',
        'Karikari',
        'Koomson',
        'Kumi',
        'Kusi',
        'Kwaakye',
        'Kwarteng',
        'Kyei',
        'Mensa',
        'Mensah',
        'Nkansa',
        'Nkansah',
        'Nkrumah',
        'Nsia',
        'Nti',
        'Ntiamoa',
        'Ntim',
        'Nyaako',
        'Nyame',
        'Nyantakyi',
        'Obeng',
        'Ofori',
        'Ofosu',
        'Okyere',
        'Omani',
        'Opoku',
        'Oppong',
        'Opuku',
        'Osei',
        'Oti',
        'Otiwa',
        'Otuo',
        'Owusu',
        'Prempeh',
        'Quartey',
        'Safo',
        'Sarpong',
        'Takyi',
        'Tawia',
        'Tutu',
        'Tweneboa',
        'Twumasi',
        'Wiafe',
        'Yaamoa',
        'Yawson',
        'Yeboa',
        'Yeboah',
        'Yirenkyi'
    )

    last_names_female = (
        'Aboraa',
        'Abrafi',
        'Acheampong',
        'Adoma',
        'Adomah',
        'Adomako',
        'Adu',
        'Adusei',
        'Adutwum',
        'Adutwumwaa',
        'Adwubi',
        'Afirifa',
        'Afoakwa',
        'Afrakomaa',
        'Agyapomaa',
        'Agyapong',
        'Agyapong',
        'Agyare',
        'Agyei',
        'Agyeiwaa',
        'Agyemang',
        'Ahortor',
        'Akoaa',
        'Akoto',
        'Akowua',
        'Akyaa',
        'Akyeamfuɔ',
        'Akyeampomaa',
        'Akyeampong',
        'Akyena',
        'Akyerε',
        'Akyerεko',
        'Akɔmaa',
        'Amo',
        'Amoa',
        'Amoako',
        'Amoakowaa',
        'Amoanimaa',
        'Amoasi',
        'Ampadu',
        'Ampofo',
        'Ampofowaa',
        'Ampoma',
        'Amponsa',
        'Amponsa',
        'Andorful',
        'Anima',
        'Ankra',
        'Anokye',
        'Ansa',
        'Ansomaa',
        'Ansomah',
        'Antwi',
        'Antwi',
        'Antwiwaa',
        'Appia',
        'Appiah',
        'Asamoa',
        'Asamoah',
        'Asante',
        'Asantewaa',
        'Asare',
        'Asenso',
        'Asiama',
        'Asiedu',
        'Asieduwaa',
        'Ata',
        'Ataa',
        'Awuah',
        'Baa',
        'Baafi',
        'Baah',
        'Baawia',
        'Badu',
        'Boadi',
        'Boadu',
        'Boahen',
        'Boakye',
        'Boakye',
        'Boakyewaa',
        'Boatemaa',
        'Boatemaah',
        'Boaten',
        'Boateng',
        'Bona',
        'Bonsra',
        'Bonsu',
        'Daako',
        'Daakoaa',
        'Danso',
        'Darko',
        'Donkor',
        'Duah',
        'Dufie',
        'Dwamena',
        'Fofie',
        'Foriwaa',
        'Fosu',
        'Fosua',
        'Frema',
        'Frimpomaa',
        'Gyamfi',
        'Gyamfi',
        'Gyamfiaa',
        'Gyasi',
        'Gyasiwaa',
        'Karikari',
        'Koomson',
        'Kumi',
        'Kusi',
        'Kusiwaa',
        'Kwaakye',
        'Kwaakyewaa',
        'Kwartemaa',
        'Kwarteng',
        'Kyei',
        'Kyeiwaa',
        'Kyerewaa',
        'Mansa',
        'Mensa',
        'Mensah',
        'Nkansa',
        'Nkansah',
        'Nkrumah',
        'Nsia',
        'Nti',
        'Ntiamoa',
        'Ntim',
        'Nyaako',
        'Nyaakoaa',
        'Nyame',
        'Nyantakyi',
        'Obeng',
        'Ofori',
        'Ofosu',
        'Okyere',
        'Okyere',
        'Omani',
        'Opoku',
        'Oppong',
        'Opuku',
        'Osei',
        'Oti',
        'Otiwa',
        'Otuo',
        'Owusu',
        'Owusuwaa',
        'Pokuaa',
        'Pomaa',
        'Prempeh',
        'Quartey',
        'Safo',
        'Safo',
        'Safoaa',
        'Sarpong',
        'Serwaa',
        'Takyi',
        'Tawia',
        'Tiwaa',
        'Tutu',
        'Tweneboa',
        'Twumasi',
        'Wiafe',
        'Yaamoa',
        'Yawson',
        'Yeboa',
        'Yeboah',
        'Yirenkyi'
    )

    last_names = last_names_male + last_names_female

    prefixes_female = ('Mrs.', 'Ms.', 'Miss', 'Dr.',
                       'Mama', 'Maame', 'Awura', 'Sista'
                       'Osofo Maame')

    prefixes_male = ('Mr.', 'Dr.', 'Agya', 'Owura', 'Osofo')



# coding=utf-8

from __future__ import unicode_literals
from __future__ import print_function

import os
import sys
import argparse

from faker import Faker, documentor
from faker import VERSION
from faker.config import AVAILABLE_LOCALES, DEFAULT_LOCALE, META_PROVIDERS_MODULES


if sys.version < '3':
    text_type = unicode
    binary_type = str
else:
    text_type = str
    binary_type = bytes


__author__ = 'joke2k'


def print_provider(doc, provider, formatters, excludes=None, output=None):
    output = output or sys.stdout
    if excludes is None:
        excludes = []

    print(file=output)
    print("### {0}".format(
          doc.get_provider_name(provider)), file=output)
    print(file=output)

    for signature, example in formatters.items():
        if signature in excludes:
            continue
        try:
            lines = text_type(example).expandtabs().splitlines()
        except UnicodeDecodeError:
            # The example is actually made of bytes.
            # We could coerce to bytes, but that would fail anyway when we wiil
            # try to `print` the line.
            lines = ["<bytes>"]
        except UnicodeEncodeError:
            raise Exception('error on "{0}" with value "{1}"'.format(
                            signature, example))
        margin = max(30, doc.max_name_len + 1)
        remains = 150 - margin
        separator = '#'
        for line in lines:
            for i in range(0, (len(line) // remains) + 1):
                print("\t{fake:<{margin}}{separator} {example}".format(
                    fake=signature,
                    separator=separator,
                    example=line[i * remains:(i + 1) * remains],
                    margin=margin
                ), file=output)
                signature = separator = ' '


def print_doc(provider_or_field=None,
              args=None, lang=DEFAULT_LOCALE, output=None, includes=None):
    args = args or []
    output = output or sys.stdout
    fake = Faker(locale=lang, includes=includes)

    from faker.providers import BaseProvider
    base_provider_formatters = [f for f in dir(BaseProvider)]

    if provider_or_field:
        if '.' in provider_or_field:
            parts = provider_or_field.split('.')
            locale = parts[-2] if parts[-2] in AVAILABLE_LOCALES else lang
            fake = Faker(locale, providers=[provider_or_field], includes=includes)
            doc = documentor.Documentor(fake)
            doc.already_generated = base_provider_formatters
            print_provider(
                doc,
                fake.get_providers()[0],
                doc.get_provider_formatters(fake.get_providers()[0]),
                output=output)
        else:
            try:
                print(fake.format(provider_or_field, *args), end='', file=output)
            except AttributeError:
                raise ValueError('No faker found for "{0}({1})"'.format(
                    provider_or_field, args))

    else:
        doc = documentor.Documentor(fake)

        formatters = doc.get_formatters(with_args=True, with_defaults=True)

        for provider, fakers in formatters:

            print_provider(doc, provider, fakers, output=output)

        for language in AVAILABLE_LOCALES:
            if language == lang:
                continue
            print(file=output)
            print('## LANGUAGE {0}'.format(language), file=output)
            fake = Faker(locale=language)
            d = documentor.Documentor(fake)

            for p, fs in d.get_formatters(with_args=True, with_defaults=True,
                                          locale=language,
                                          excludes=base_provider_formatters):
                print_provider(d, p, fs, output=output)


class Command(object):

    def __init__(self, argv=None):
        self.argv = argv or sys.argv[:]
        self.prog_name = os.path.basename(self.argv[0])

    def execute(self):
        """
        Given the command-line arguments, this creates a parser appropriate
        to that command, and runs it.
        """

        # retrieve default language from system environment
        default_locale = os.environ.get('LANG', 'en_US').split('.')[0]
        if default_locale not in AVAILABLE_LOCALES:
            default_locale = DEFAULT_LOCALE

        epilog = """supported locales:

  {0}

  faker can take a locale as an argument, to return localized data. If no
  localized provider is found, the factory falls back to the default en_US
  locale.

examples:

  $ faker address
  968 Bahringer Garden Apt. 722
  Kristinaland, NJ 09890

  $ faker -l de_DE address
  Samira-Niemeier-Allee 56
  94812 Biedenkopf

  $ faker profile ssn,birthdate
  {{'ssn': u'628-10-1085', 'birthdate': '2008-03-29'}}

  $ faker -r=3 -s=";" name
  Willam Kertzmann;
  Josiah Maggio;
  Gayla Schmitt;

""".format(', '.join(sorted(AVAILABLE_LOCALES)))

        formatter_class = argparse.RawDescriptionHelpFormatter
        parser = argparse.ArgumentParser(
            prog=self.prog_name,
            description='{0} version {1}'.format(self.prog_name, VERSION),
            epilog=epilog,
            formatter_class=formatter_class)

        parser.add_argument("--version", action="version",
                            version="%(prog)s {0}".format(VERSION))

        parser.add_argument('-o', metavar="output",
                            type=argparse.FileType('w'),
                            default=sys.stdout,
                            help="redirect output to a file")

        parser.add_argument('-l', '--lang',
                            choices=AVAILABLE_LOCALES,
                            default=default_locale,
                            metavar='LOCALE',
                            help="specify the language for a localized "
                            "provider (e.g. de_DE)")
        parser.add_argument('-r', '--repeat',
                            default=1,
                            type=int,
                            help="generate the specified number of outputs")
        parser.add_argument('-s', '--sep',
                            default='\n',
                            help="use the specified separator after each "
                            "output")

        parser.add_argument('-i',
                            '--include',
                            default=META_PROVIDERS_MODULES,
                            nargs='*',
                            help="list of additional custom providers to "
                            "user, given as the import path of the module "
                            "containing your Provider class (not the provider "
                            "class itself)")

        parser.add_argument('fake',
                            action='store',
                            nargs='?',
                            help="name of the fake to generate output for "
                                 "(e.g. profile)")

        parser.add_argument('fake_args',
                            metavar="fake argument",
                            action='store',
                            nargs='*',
                            help="optional arguments to pass to the fake "
                                 "(e.g. the profile fake takes an optional "
                                 "list of comma separated field names as the "
                                 "first argument)")

        arguments = parser.parse_args(self.argv[1:])

        for i in range(arguments.repeat):

            print_doc(arguments.fake,
                      arguments.fake_args,
                      lang=arguments.lang,
                      output=arguments.o,
                      includes=arguments.include
                      )
            print(arguments.sep, file=arguments.o)

            if not arguments.fake:
                # repeat not supported for all docs
                break


def execute_from_command_line(argv=None):
    """A simple method that runs a Command."""
    if sys.stdout.encoding is None:
        print('please set python env PYTHONIOENCODING=UTF-8, example: '
              'export PYTHONIOENCODING=UTF-8, when writing to stdout',
              file=sys.stderr)
        exit(1)

    command = Command(argv)
    command.execute()
